import { Record } from 'immutable';
import isPlainObject from 'is-plain-object';
import { Mark, Data } from 'slate';
import { hasMark, removeCollapsedMark, removeExpandedMark } from '@vericus/slate-kit-plugins-utils';
import tinycolor from 'tinycolor2';
import React from 'react';

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

var toConsumableArray = function (arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  } else {
    return Array.from(arr);
  }
};

var defaultOptions = {
  type: undefined,
  styles: undefined,
  data: undefined,
  defaultColor: undefined,
  alpha: 1
};

var Options = function (_Record) {
  inherits(Options, _Record);

  function Options() {
    classCallCheck(this, Options);
    return possibleConstructorReturn(this, (Options.__proto__ || Object.getPrototypeOf(Options)).apply(this, arguments));
  }

  createClass(Options, [{
    key: "toJS",


    /**
     * Alias `toJS`.
     */

    value: function toJS(options) {
      return this.toJSON(options);
    }
  }], [{
    key: "create",
    value: function create() {
      var attrs = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

      if (Options.isOptions(attrs)) return attrs;
      if (isPlainObject(attrs)) return Options.fromJSON(attrs);

      throw new Error("`Options.create` only accepts objects, strings or options");
    }
  }, {
    key: "fromJSON",
    value: function fromJSON(object) {
      if (Options.isOptions(object)) return object;

      var type = object.type,
          data = object.data,
          styles = object.styles,
          defaultColor = object.defaultColor,
          _object$alpha = object.alpha,
          alpha = _object$alpha === undefined ? 1 : _object$alpha;

      if (!type || !data || !styles || !defaultColor || !Array.isArray(styles)) {
        throw new Error("Highlight text require type, data and style option");
      }

      if (alpha < 0 || alpha > 1) {
        throw new Error("Alpha value can only go from 0 to 1");
      }

      return new Options({
        type: type,
        data: data,
        styles: styles,
        alpha: alpha,
        defaultColor: defaultColor
      });
    }
  }, {
    key: "isOptions",
    value: function isOptions(any) {
      return !!(any instanceof Record && ["type", "data", "styles", "defaultColor"].every(function (key) {
        return any.has(key);
      }));
    }
  }]);
  return Options;
}(Record(defaultOptions, "highlight text option"));

function createChanges(opt) {
  var defaultColor = opt.defaultColor,
      data = opt.data,
      type = opt.type;

  return {
    changeColor: function changeColor(change, color) {
      var value = change.value;
      var selection = value.selection,
          document = value.document;

      if (hasMark(value, type)) {
        if (tinycolor(color).toName() === defaultColor) {
          if (selection.isCollapsed) {
            removeCollapsedMark(value, change, type);
          } else {
            removeExpandedMark(document, selection, change, type);
          }
        } else {
          var newMark = new Mark({
            type: type,
            data: Data.create(defineProperty({}, data, color))
          });
          if (selection.isCollapsed) {
            removeCollapsedMark(value, change, type);
            change.focus();
            change.addMark(newMark);
          } else {
            removeExpandedMark(document, selection, change, type);
            change.addMarkAtRange(selection, newMark);
          }
        }
      } else if (tinycolor(color).toName() !== defaultColor) {
        var mark = new Mark({
          type: type,
          data: Data.create(defineProperty({}, data, color))
        });
        if (selection.isCollapsed) {
          change.addMark(mark);
        } else {
          removeExpandedMark(document, selection, change, type);
          change.addMarkAtRange(selection, mark);
        }
      }
      change.focus();
      return change;
    }
  };
}

var defaultStyle = {
  textDecoration: "inherit",
  textDecorationColor: "inherit"
};

function createRenderMark(opt) {
  var type = opt.type,
      dataField = opt.data,
      styleFields = opt.styles,
      alpha = opt.alpha;

  return function (props) {
    if (props.mark.type === type) {
      var _children = props.children,
          _attributes = props.attributes,
          data = props.mark.data;

      var _ref = _attributes || {},
          style = _ref.style;

      var color = tinycolor(data.get(dataField));
      style = _extends({}, defaultStyle, style, {
        textDecoration: "inherit",
        decorationStyle: "initial"
      }, styleFields.reduce(function (acc, styleField) {
        return _extends({}, acc, defineProperty({}, styleField, color.setAlpha(alpha).toRgbString()));
      }, {}));
      return React.createElement(
        "span",
        _extends({}, _attributes, { style: style }),
        _children
      );
    }
    return undefined;
  };
}

function createUtils(opts) {
  var defaultColor = opts.defaultColor,
      data = opts.data,
      type = opts.type;

  return {
    currentColor: function currentColor(value) {
      if (value.activeMarks) {
        var activeColorMarks = value.activeMarks.filter(function (mark) {
          return mark.type === type;
        }).map(function (mark) {
          return mark.data.get(data);
        });
        var uniqueColorMarks = [].concat(toConsumableArray(new Set(activeColorMarks)));
        if (uniqueColorMarks.length !== 0) {
          return uniqueColorMarks[0];
        }
      }
      return defaultColor;
    }
  };
}

function createSchema(opts) {
  var type = opts.type,
      data = opts.data;

  var schemas = {};
  var schema = {
    document: {
      marks: [{
        type: type,
        data: defineProperty({}, data, function (e) {
          return !e || tinycolor(e).isValid();
        })
      }]
    }
  };
  schemas.getSchema = function () {
    return schema;
  };
  return schemas;
}

function createPlugin(pluginOptions) {
  var options = Options.create(pluginOptions);
  var changes = createChanges(options);
  var renderMark = createRenderMark(options);
  var schemas = createSchema(options);
  var utils = createUtils(options);
  return _extends({ options: options, changes: changes, utils: utils, renderMark: renderMark }, schemas);
}

export default createPlugin;
//# sourceMappingURL=slate-kit-highlight-text.es.js.map
